package com.ruoyi.zdsz.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.annotation.Version;
import com.ruoyi.common.annotation.DynamicName;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.core.domain.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ruoyi.zdsz.domain.bo.ZEngineeringInfoBo;
import com.ruoyi.zdsz.service.IZEngineeringInfoService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import com.ruoyi.zdsz.domain.bo.ZEngineeringNodeBo;
import com.ruoyi.zdsz.domain.vo.ZEngineeringNodeVo;
import com.ruoyi.zdsz.domain.ZEngineeringNode;
import com.ruoyi.zdsz.mapper.ZEngineeringNodeMapper;
import com.ruoyi.zdsz.service.IZEngineeringNodeService;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Collection;

/**
 * 工程节点Service业务层处理
 *
 * @author ruoyi
 * @date 2023-12-27
 */
@RequiredArgsConstructor
@Service
public class ZEngineeringNodeServiceImpl implements IZEngineeringNodeService {

    private final ZEngineeringNodeMapper baseMapper;
    @Resource
    private IZEngineeringInfoService izEngineeringInfoService;
    /**
     * 查询工程节点
     */
    @Override
    public ZEngineeringNodeVo queryById(String id){
        return baseMapper.selectVoById(id);
    }

    @Override
    @DynamicName(spel = "#createTime")
    public ZEngineeringNodeBo query(String id,Date createTime) {
        ZEngineeringNode zEngineeringNodeVo=baseMapper.selectById(id);
        ZEngineeringNodeBo zEngineeringNodeBo= BeanUtil.toBean(zEngineeringNodeVo, ZEngineeringNodeBo.class);
       ZEngineeringInfoBo zEngineeringInfoBo= izEngineeringInfoService.query(zEngineeringNodeBo);
       zEngineeringNodeBo.setzEngineeringInfoBo(zEngineeringInfoBo);
        return zEngineeringNodeBo;
    }

    /**
     * 查询工程节点列表
     */
    @Override
    public TableDataInfo<ZEngineeringNodeVo> queryPageList(ZEngineeringNodeBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<ZEngineeringNode> lqw = buildQueryWrapper(bo);
        Page<ZEngineeringNodeVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询工程节点列表
     */
    @Override
    public List<ZEngineeringNodeVo> queryList(ZEngineeringNodeBo bo) {
        LambdaQueryWrapper<ZEngineeringNode> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<ZEngineeringNode> buildQueryWrapper(ZEngineeringNodeBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<ZEngineeringNode> lqw = Wrappers.lambdaQuery();
        lqw.eq(StringUtils.isNotBlank(bo.getCivliId()), ZEngineeringNode::getCivliId, bo.getCivliId());
        lqw.eq(StringUtils.isNotBlank(bo.getType()), ZEngineeringNode::getType, bo.getType());
        return lqw;
    }

    /**
     * 新增工程节点
     */
    @Override

    public Boolean insertByBo(ZEngineeringNodeBo bo) {
        ZEngineeringNode add = BeanUtil.toBean(bo, ZEngineeringNode.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        return flag;
    }
    /**
     * 新增工程节点
     * @param bo
     * @return ZEngineeringNode
     */
    @Override
    @Transactional(rollbackFor=Exception.class)
    @DynamicName(spel = "#bo.createTime")
    public ZEngineeringNode insert(ZEngineeringNodeBo bo) {
        ZEngineeringNode add = BeanUtil.toBean(bo, ZEngineeringNode.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        bo.getzEngineeringInfoBo().setEngInfoId(add.getId());
        bo.getzEngineeringInfoBo().setCreateTime(bo.getCreateTime());
        izEngineeringInfoService.insert(bo.getzEngineeringInfoBo());
        return add;
    }
    /**
     * 修改工程节点
     */
    @Override
    public Boolean updateByBo(ZEngineeringNodeBo bo) {
        ZEngineeringNode update = BeanUtil.toBean(bo, ZEngineeringNode.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    @Override
    @DynamicName(spel = "#bo.createTime")
    @Transactional(rollbackFor = Exception.class)
    public Boolean update(ZEngineeringNodeBo bo) {
        ZEngineeringNode update = BeanUtil.toBean(bo, ZEngineeringNode.class);
        validEntityBeforeSave(update);
        izEngineeringInfoService.updateByBo(bo.getzEngineeringInfoBo());
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(ZEngineeringNode entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 批量删除工程节点
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<String> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteBatchIds(ids) > 0;
    }
}
